@@ -61,21 +61,20 @@ mod transaction;
 
 use cache::{CacheTracker, NoopCacheTracker};
 use db::*;
+use error::Context;
 pub use error::{BlockStoreError, Result};
-use libipld::{cid::Cid, codec::References, store::StoreParams, Block, Ipld};
+use libipld::{codec::References, store::StoreParams, Block, Cid, Ipld};
 use parking_lot::Mutex;
 use rusqlite::{Connection, DatabaseName, OpenFlags};
 use std::{
+    collections::HashSet,
     fmt,
     iter::FromIterator,
     marker::PhantomData,
     mem,
     ops::DerefMut,
     path::{Path, PathBuf},
-    sync::{
-        atomic::{AtomicI64, Ordering},
-        Arc,
-    },
+    sync::Arc,
     time::Duration,
 };
 use tracing::*;
@@ -158,7 +157,7 @@ impl fmt::Display for Synchronous {
     }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Config {
     size_targets: SizeTargets,
     cache_tracker: Arc<dyn CacheTracker>,
@@ -189,8 +188,8 @@ impl Config {
         self
     }
     /// Set size targets for the store
-    pub fn with_size_targets(mut self, size_targets: SizeTargets) -> Self {
-        self.size_targets = size_targets;
+    pub fn with_size_targets(mut self, count: u64, size: u64) -> Self {
+        self.size_targets = SizeTargets { count, size };
         self
     }
     /// Set strategy for which non-pinned blocks to keep in case one of the size targets is exceeded.
@@ -212,6 +211,7 @@ pub struct BlockStore<S> {
     conn: Connection,
     expired_temp_pins: Arc<Mutex<Vec<i64>>>,
     config: Config,
+    db_path: DbPath,
     _s: PhantomData<S>,
 }
 
@@ -233,22 +233,32 @@ impl StoreStats {
     }
 }
 
-// do not implement Clone for this!
 /// a handle that contains a temporary pin
 ///
-/// dropping this handle enqueue the pin for dropping before the next gc.
+/// Dropping this handle enqueues the pin for dropping before the next gc.
+// do not implement Clone for this!
 pub struct TempPin {
-    id: AtomicI64,
+    id: i64,
     expired_temp_pins: Arc<Mutex<Vec<i64>>>,
 }
 
+impl TempPin {
+    fn new(expired_temp_pins: Arc<Mutex<Vec<i64>>>) -> Self {
+        Self {
+            id: 0,
+            expired_temp_pins,
+        }
+    }
+}
+
 /// dump the temp alias id so you can find it in the database
 impl fmt::Debug for TempPin {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let id = self.id.load(Ordering::SeqCst);
         let mut builder = f.debug_struct("TempAlias");
-        if id > 0 {
-            builder.field("id", &id);
+        if self.id > 0 {
+            builder.field("id", &self.id);
+        } else {
+            builder.field("unused", &true);
         }
         builder.finish()
     }
@@ -256,12 +266,8 @@ impl fmt::Debug for TempPin {
 
 impl Drop for TempPin {
     fn drop(&mut self) {
-        let id = self.id.get_mut();
-        let alias = *id;
-        if alias > 0 {
-            // not sure if we have to guard against double drop, but it certainly does not hurt.
-            *id = 0;
-            self.expired_temp_pins.lock().push(alias);
+        if self.id > 0 {
+            self.expired_temp_pins.lock().push(self.id);
         }
     }
 }
@@ -282,27 +288,74 @@ where
             flags |= OpenFlags::SQLITE_OPEN_CREATE
         }
         let conn = match db_path {
-            DbPath::Memory => Connection::open_in_memory()?,
-            DbPath::File(path) => Connection::open_with_flags(path, flags)?,
+            DbPath::Memory => Connection::open_in_memory().ctx("opening in-memory DB")?,
+            DbPath::File(path) => Connection::open_with_flags(path, flags).ctx("opening DB")?,
         };
         Ok(conn)
     }
 
     pub fn open_path(db_path: DbPath, config: Config) -> crate::Result<Self> {
         let is_memory = db_path.is_memory();
-        let mut conn = Self::create_connection(db_path, &config)?;
+        let mut conn = Self::create_connection(db_path.clone(), &config)?;
+        // this needs to be done only once, and before the first transaction
+        conn.execute_batch("PRAGMA journal_mode = WAL")
+            .ctx("setting WAL mode")?;
         init_db(
             &mut conn,
             is_memory,
             config.pragma_cache_pages as i64,
             config.pragma_synchronous,
         )?;
-        let ids = in_txn(&mut conn, |txn| get_ids(txn))?;
-        config.cache_tracker.retain_ids(&ids);
-        Ok(Self {
+        let mut this = Self {
             conn,
             expired_temp_pins: Arc::new(Mutex::new(Vec::new())),
             config,
+            db_path,
+            _s: PhantomData,
+        };
+        if !is_memory {
+            let mut conn = this.additional_connection()?;
+            std::thread::spawn(move || {
+                if let Err(e) = recompute_store_stats(&mut conn.conn) {
+                    tracing::error!("cannot recompute store stats: {}", e);
+                }
+            });
+        }
+        if this.config.cache_tracker.has_persistent_state() {
+            let ids = in_txn(
+                &mut this.conn,
+                Some(("get IDs", Duration::from_secs(1))),
+                get_ids,
+            )?;
+            this.config.cache_tracker.retain_ids(&ids);
+        }
+        Ok(this)
+    }
+
+    /// Create another connection to the underlying database
+    ///
+    /// This allows you to perform operations in parallel.
+    pub fn additional_connection(&self) -> crate::Result<Self> {
+        if self.db_path.is_memory() {
+            return Err(BlockStoreError::NoAdditionalInMemory);
+        }
+        let mut conn = Self::create_connection(self.db_path.clone(), &self.config)?;
+        init_pragmas(
+            &mut conn,
+            self.db_path.is_memory(),
+            self.config.pragma_cache_pages as i64,
+        )?;
+        conn.pragma_update(
+            None,
+            "synchronous",
+            &self.config.pragma_synchronous.to_string(),
+        )
+        .ctx("setting synchronous mode")?;
+        Ok(Self {
+            conn,
+            expired_temp_pins: self.expired_temp_pins.clone(),
+            config: self.config.clone(),
+            db_path: self.db_path.clone(),
             _s: PhantomData,
         })
     }
@@ -342,239 +395,202 @@ where
                     debug!("Restoring: {} %", percent);
                 }
             }),
+        )
+        .ctx("restoring test DB from backup")?;
+        let ids = in_txn(
+            &mut conn,
+            Some(("get ids", Duration::from_secs(1))),
+            get_ids,
         )?;
-        let ids = in_txn(&mut conn, |txn| get_ids(txn))?;
         config.cache_tracker.retain_ids(&ids);
         Ok(Self {
             conn,
             expired_temp_pins: Arc::new(Mutex::new(Vec::new())),
             config,
+            db_path: DbPath::Memory,
             _s: PhantomData,
         })
     }
 
-    pub fn flush(&self) -> crate::Result<()> {
-        // TODO: check if this works! We are always in WAL mode.
-        // https://www.sqlite.org/pragma.html#pragma_wal_checkpoint
-        Ok(self.conn.pragma_update(None, "wal_checkpoint", &"FULL")?)
+    pub fn backup(&mut self, path: impl AsRef<Path>) -> Result<()> {
+        in_txn(&mut self.conn, None, move |txn| {
+            txn.backup(DatabaseName::Main, path.as_ref(), None)
+                .ctx("backing up DB")
+        })
     }
 
-    pub fn integrity_check(&self) -> crate::Result<()> {
-        let result = integrity_check(&self.conn)?;
+    pub fn flush(&mut self) -> crate::Result<()> {
+        in_txn(&mut self.conn, None, |txn| {
+            txn.pragma_update(None, "wal_checkpoint", &"TRUNCATE")
+                .ctx("flushing WAL")
+        })
+    }
+
+    pub fn integrity_check(&mut self) -> crate::Result<()> {
+        let result = integrity_check(&mut self.conn)?;
         if result == vec!["ok".to_owned()] {
             Ok(())
         } else {
             let error_text = result.join(";");
             Err(crate::error::BlockStoreError::SqliteError(
                 rusqlite::Error::SqliteFailure(rusqlite::ffi::Error::new(11), Some(error_text)),
+                "checking integrity",
             ))
         }
+        // FIXME add actual integrity check on the stored blocks
     }
 
-    pub fn transaction(&mut self) -> Result<Transaction<'_, S>> {
+    pub fn transaction(&mut self) -> Transaction<'_, S> {
         Transaction::new(self)
     }
 
     /// Get a temporary alias for safely adding blocks to the store
     pub fn temp_pin(&self) -> TempPin {
-        TempPin {
-            id: AtomicI64::new(0),
-            expired_temp_pins: self.expired_temp_pins.clone(),
-        }
+        TempPin::new(self.expired_temp_pins.clone())
     }
 
-    /// Add a permanent named alias/pin for a root
-    pub fn alias(&mut self, name: impl AsRef<[u8]>, link: Option<&Cid>) -> crate::Result<()> {
-        let txn = self.transaction()?;
-        txn.alias(name, link)?;
-        txn.commit()
-    }
-
-    /// Resolves an alias to a cid.
-    pub fn resolve(&mut self, name: impl AsRef<[u8]>) -> crate::Result<Option<Cid>> {
-        self.transaction()?.resolve(name)
-    }
-
-    pub fn extend_temp_pin(
-        &mut self,
-        pin: &TempPin,
-        links: impl IntoIterator<Item = Cid>,
-    ) -> crate::Result<()> {
-        let txn = self.transaction()?;
-        for link in links {
-            txn.extend_temp_pin(pin, &link)?;
-        }
-        txn.commit()
-    }
-
-    /// Returns the aliases referencing a block.
-    pub fn reverse_alias(&mut self, cid: &Cid) -> crate::Result<Option<Vec<Vec<u8>>>> {
-        self.transaction()?.reverse_alias(cid)
-    }
-
-    /// Checks if the store knows about the cid.
-    /// Note that this does not necessarily mean that the store has the data for the cid.
-    pub fn has_cid(&mut self, cid: &Cid) -> Result<bool> {
-        self.transaction()?.has_cid(cid)
-    }
-
-    /// Checks if the store has the data for a cid
-    pub fn has_block(&mut self, cid: &Cid) -> Result<bool> {
-        self.transaction()?.has_block(cid)
-    }
-
-    /// Get the stats for the store.
+    /// Run a full VACUUM on the SQLITE database
     ///
-    /// The stats are kept up to date, so this is fast.
-    pub fn get_store_stats(&mut self) -> Result<StoreStats> {
-        self.transaction()?.get_store_stats()
-    }
-
-    /// Get all cids that the store knows about
-    pub fn get_known_cids<C: FromIterator<Cid>>(&mut self) -> Result<C> {
-        self.transaction()?.get_known_cids()
-    }
-
-    /// Get all cids for which the store has blocks
-    pub fn get_block_cids<C: FromIterator<Cid>>(&mut self) -> Result<C> {
-        self.transaction()?.get_block_cids()
-    }
-
-    /// Get descendants of a cid
-    pub fn get_descendants<C: FromIterator<Cid>>(&mut self, cid: &Cid) -> Result<C> {
-        self.transaction()?.get_descendants(cid)
-    }
-
-    /// Given a root of a dag, gives all cids which we do not have data for.
-    pub fn get_missing_blocks<C: FromIterator<Cid>>(&mut self, cid: &Cid) -> Result<C> {
-        self.transaction()?.get_missing_blocks(cid)
-    }
-
-    /// list all aliases
-    pub fn aliases<C: FromIterator<(Vec<u8>, Cid)>>(&mut self) -> Result<C> {
-        self.transaction()?.aliases()
-    }
-
-    /// Add a number of blocks to the store
-    ///
-    /// - `blocks` the blocks to add.
-    /// - `alias` an optional temporary alias.
-    ///   This can be used to incrementally add blocks without having to worry about them being garbage
-    ///   collected before they can be pinned with a permanent alias.
-    pub fn put_blocks(
-        &mut self,
-        blocks: impl IntoIterator<Item = Block<S>>,
-        pin: Option<&TempPin>,
-    ) -> Result<()> {
-        let mut txn = self.transaction()?;
-        for block in blocks {
-            txn.put_block(&block, pin)?;
-        }
-        txn.commit()
-    }
-
-    /// Add a single block
-    ///
-    /// this is just a convenience method that calls put_blocks internally.
-    ///
-    /// - `cid` the cid
-    ///   This should be a hash of the data, with some format specifier.
-    /// - `data` a blob
-    /// - `links` links extracted from the data
-    /// - `alias` an optional temporary alias
-    pub fn put_block(&mut self, block: &Block<S>, pin: Option<&TempPin>) -> Result<()> {
-        let mut txn = self.transaction()?;
-        txn.put_block(block, pin)?;
-        txn.commit()
-    }
-
-    /// Get data for a block
-    ///
-    /// Will return None if we don't have the data
-    pub fn get_block(&mut self, cid: &Cid) -> Result<Option<Vec<u8>>> {
-        self.transaction()?.get_block(cid)
-    }
-
+    /// This may take a while, blocking all other writes to the store.
     pub fn vacuum(&mut self) -> Result<()> {
-        vacuum(&self.conn)
+        vacuum(&mut self.conn)
     }
 
-    /// do a full garbage collection
+    /// Perform maintenance on the TempPins
     ///
-    /// for a large block store, this can take several seconds to minutes. If that is not acceptable,
-    /// consider using incremental gc.
+    /// This is done automatically upon every (incremental) GC, so you normally don’t need to call this.
+    pub fn cleanup_temp_pins(&mut self) -> Result<()> {
+        // atomically grab the expired_temp_pins until now
+        let expired_temp_pins = mem::take(self.expired_temp_pins.lock().deref_mut());
+        in_txn(
+            &mut self.conn,
+            Some(("dropping expired temp_pins", Duration::from_millis(100))),
+            move |txn| {
+                // get rid of dropped temp aliases, this should be fast
+                for id in expired_temp_pins.iter() {
+                    delete_temp_pin(txn, *id)?;
+                }
+                Ok(())
+            },
+        )
+    }
+
+    /// Perform full GC
+    ///
+    /// This is the same as running incremental GC without limits, plus a full SQLITE VACUUM.
     pub fn gc(&mut self) -> Result<()> {
-        loop {
-            let complete = self.incremental_gc(20000, Duration::from_secs(1))?;
-            while !self.incremental_delete_orphaned(20000, Duration::from_secs(1))? {}
-            if complete {
-                break;
-            }
-        }
+        self.cleanup_temp_pins()?;
+        incremental_gc(
+            &mut self.conn,
+            usize::MAX,
+            Duration::from_secs(u32::MAX.into()),
+            self.config.size_targets,
+            &self.config.cache_tracker,
+        )?;
+        self.vacuum()?;
         Ok(())
     }
+
     /// Perform an incremental garbage collection.
     ///
     /// Will collect unpinned blocks until either the size targets are met again, or at minimum
     /// `min_blocks` blocks are collected. Then it will continue connecting blocks until `max_duration`
     /// is elapsed.
     ///
-    /// Note that this might significantly exceed `max_duration` for various reasons. Also note that
-    /// when doing incremental gc, the actual blocks are not yet deleted. So a call to this method
-    /// should usually be followed by a call to incremental_delete_orphaned.
-    ///
-    /// - `min_blocks` the minium number of blocks to collect in any case
-    /// - `max_duration` the maximum duration that should be spent on gc
+    /// Note that this might significantly exceed `max_duration` for various reasons.
     ///
     /// Returns true if either size targets are met or there are no unpinned blocks left.
     pub fn incremental_gc(&mut self, min_blocks: usize, max_duration: Duration) -> Result<bool> {
-        // atomically grab the expired_temp_pins until now
-        let expired_temp_pins = {
-            let mut result = Vec::new();
-            mem::swap(self.expired_temp_pins.lock().deref_mut(), &mut result);
-            result
-        };
-        let (deleted, complete) = log_execution_time("gc", Duration::from_secs(1), || {
-            let size_targets = self.config.size_targets;
-            let cache_tracker = &self.config.cache_tracker;
-            in_txn(&mut self.conn, move |txn| {
-                // get rid of dropped temp aliases, this should be fast
-                for id in expired_temp_pins {
-                    delete_temp_pin(txn, id)?;
-                }
-                incremental_gc(txn, min_blocks, max_duration, size_targets, cache_tracker)
-            })
+        self.cleanup_temp_pins()?;
+        let ret = incremental_gc(
+            &mut self.conn,
+            min_blocks,
+            max_duration,
+            self.config.size_targets,
+            &self.config.cache_tracker,
+        )?;
+        in_txn(&mut self.conn, None, |txn| {
+            txn.execute_batch("PRAGMA incremental_vacuum")
+                .ctx("incremental vacuum")
         })?;
-        self.config.cache_tracker.blocks_deleted(deleted);
-        Ok(complete)
+        Ok(ret)
     }
-    /// Incrementally delete orphaned blocks
-    ///
-    /// Orphaned blocks are blocks for which we have deleted the metadata in `incremental_gc`.
-    ///
-    /// Will delete orphaned blocks until either all orphaned blocks are deleted, or at minimum
-    /// `min_blocks` blocks are deleted. Then it will continue deleting blocks until `max_duration`
-    /// is elapsed.
-    ///
-    /// Note that this might significantly exceed `max_duration` for various reasons.
-    ///
-    /// - `min_blocks` the minium number of blocks to delete in any case
-    /// - `max_duration` the maximum duration that should be spent on gc
-    ///
-    /// Returns true if all orphaned blocks are deleted
-    pub fn incremental_delete_orphaned(
-        &mut self,
-        min_blocks: usize,
-        max_duration: Duration,
-    ) -> Result<bool> {
-        log_execution_time("delete_orphaned", Duration::from_millis(100), || {
-            let result = in_txn(&mut self.conn, move |txn| {
-                Ok(incremental_delete_orphaned(txn, min_blocks, max_duration)?)
-            })?;
-            // in tests this doesn’t return results, but in Actyx it raises ExecuteReturnedResults
-            // so we just ignore the outcome
-            self.conn.execute("PRAGMA incremental_vacuum;", []).ok();
-            Ok(result)
-        })
+}
+
+macro_rules! delegate {
+    ($($(#[$attr:meta])*$n:ident$(<$v:ident : $vt:path>)?($($arg:ident : $typ:ty),*) -> $ret:ty;)+) => {
+        $(
+            $(#[$attr])*
+            pub fn $n$(<$v: $vt>)?(&mut self, $($arg: $typ),*) -> $ret {
+                self.transaction().$n($($arg),*)
+            }
+        )+
+    };
+}
+
+impl<S> BlockStore<S>
+where
+    S: StoreParams,
+    Ipld: References<S::Codecs>,
+{
+    delegate! {
+        /// Set or delete an alias
+        alias(name: impl AsRef<[u8]>, link: Option<&Cid>) -> Result<()>;
+
+        /// Returns the aliases referencing a cid
+        reverse_alias(cid: &Cid) -> Result<Option<HashSet<Vec<u8>>>>;
+
+        /// Resolves an alias to a cid
+        resolve(name: impl AsRef<[u8]>) -> Result<Option<Cid>>;
+
+        /// Extend temp pin with an additional cid
+        extend_temp_pin(pin: &mut TempPin, link: &Cid) -> Result<()>;
+
+        /// Checks if the store knows about the cid.
+        ///
+        /// Note that this does not necessarily mean that the store has the data for the cid.
+        has_cid(cid: &Cid) -> Result<bool>;
+
+        /// Checks if the store has the data for a cid
+        has_block(cid: &Cid) -> Result<bool>;
+
+        /// Get all cids that the store knows about
+        get_known_cids<C: FromIterator<Cid>>() -> Result<C>;
+
+        /// Get all cids for which the store has blocks
+        get_block_cids<C: FromIterator<Cid>>() -> Result<C>;
+
+        /// Get descendants of a cid
+        get_descendants<C: FromIterator<Cid>>(cid: &Cid) -> Result<C>;
+
+        /// Given a root of a dag, gives all cids which we do not have data for.
+        get_missing_blocks<C: FromIterator<Cid>>(cid: &Cid) -> Result<C>;
+
+        /// list all aliases
+        aliases<C: FromIterator<(Vec<u8>, Cid)>>() -> Result<C>;
+
+        /// Put a block
+        ///
+        /// This will only be completed once the transaction is successfully committed.
+        put_block(block: &Block<S>, pin: Option<&mut TempPin>) -> Result<()>;
+
+        /// Get a block
+        get_block(cid: &Cid) -> Result<Option<Vec<u8>>>;
+
+        /// Get the stats for the store
+        ///
+        /// The stats are kept up to date, so this is fast.
+        get_store_stats() -> Result<StoreStats>;
+    }
+
+    pub fn put_blocks<I>(&mut self, blocks: I, mut pin: Option<&mut TempPin>) -> Result<()>
+    where
+        I: IntoIterator<Item = Block<S>>,
+    {
+        let mut txn = self.transaction();
+        for block in blocks {
+            txn.put_block(&block, pin.as_deref_mut())?;
+        }
+        txn.commit()
     }
 }
