@@ -12,6 +12,7 @@ use libipld::{
     multihash::{Code, MultihashDigest},
 };
 use libipld::{prelude::*, DagCbor};
+use maplit::hashset;
 use rusqlite::{params, Connection};
 use std::time::Duration;
 use tempdir::TempDir;
@@ -108,40 +109,48 @@ fn pinned(i: usize) -> Block {
 #[test]
 fn insert_get() -> anyhow::Result<()> {
     let mut store = BlockStore::memory(Config::default())?;
-    let b = block("b");
-    let c = block("c");
-    let a = links("a", vec![&b, &c, &c]);
-    store.put_block(&a, None)?;
-    // we should have all three cids
-    assert!(store.has_cid(a.cid())?);
-    assert!(store.has_cid(b.cid())?);
-    assert!(store.has_cid(c.cid())?);
-    // but only the first block
-    assert!(store.has_block(a.cid())?);
-    assert!(!store.has_block(b.cid())?);
-    assert!(!store.has_block(c.cid())?);
-    // check the data
-    assert_eq!(store.get_block(a.cid())?, Some(a.data().to_vec()));
-    // check descendants
-    assert_eq!(
-        store.get_descendants::<Vec<_>>(a.cid())?,
-        vec![*a.cid(), *b.cid(), *c.cid()]
-    );
-    // check missing blocks - should be b and c
-    assert_eq!(
-        store.get_missing_blocks::<Vec<_>>(a.cid())?,
-        vec![*b.cid(), *c.cid()]
-    );
-    // alias the root
-    store.alias(b"alias1", Some(a.cid()))?;
-    store.gc()?;
-    // after gc, we shold still have the block
-    assert!(store.has_block(a.cid())?);
-    store.alias(b"alias1", None)?;
-    store.gc()?;
-    // after gc, we shold no longer have the block
-    assert!(!store.has_block(a.cid())?);
-    Ok(())
+    let t = |store: &mut BlockStore| {
+        let b = block("b");
+        let c = block("c");
+        let a = links("a", vec![&b, &c, &c]);
+        store.put_block(&a, None)?;
+        // we should have all three cids
+        anyhow::ensure!(store.has_cid(a.cid())?);
+        anyhow::ensure!(store.has_cid(b.cid())?);
+        anyhow::ensure!(store.has_cid(c.cid())?);
+        // but only the first block
+        anyhow::ensure!(store.has_block(a.cid())?);
+        anyhow::ensure!(!store.has_block(b.cid())?);
+        anyhow::ensure!(!store.has_block(c.cid())?);
+        // check the data
+        anyhow::ensure!(store.get_block(a.cid())? == Some(a.data().to_vec()));
+        // check descendants
+        anyhow::ensure!(
+            store.get_descendants::<Vec<_>>(a.cid())? == vec![*a.cid(), *b.cid(), *c.cid()]
+        );
+        // check missing blocks - should be b and c
+        anyhow::ensure!(store.get_missing_blocks::<Vec<_>>(a.cid())? == vec![*b.cid(), *c.cid()]);
+        // alias the root
+        store.alias(b"alias1", Some(a.cid()))?;
+        store.gc()?;
+        // after gc, we shold still have the block
+        anyhow::ensure!(store.has_block(a.cid())?);
+        store.alias(b"alias1", None)?;
+        store.gc()?;
+        // after gc, we shold no longer have the block
+        anyhow::ensure!(!store.has_block(a.cid())?);
+        Ok(())
+    };
+
+    match t(&mut store) {
+        Ok(_) => Ok(()),
+        Err(e) => {
+            store
+                .backup(concat!("/tmp/inspect-", line!(), ".sqlite"))
+                .unwrap();
+            Err(e)
+        }
+    }
 }
 
 #[test]
@@ -208,7 +217,7 @@ fn size_targets() -> anyhow::Result<()> {
     // create a store with a non-empty size target to enable keeping non-pinned stuff around
     let mut store = BlockStore::memory(
         Config::default()
-            .with_size_targets(SizeTargets::new(10, 10000))
+            .with_size_targets(10, 10000)
             .with_cache_tracker(SortByIdCacheTracker),
     )?;
 
@@ -271,7 +280,7 @@ fn cache_test(tracker: impl CacheTracker + 'static) -> anyhow::Result<()> {
     // create a store with a non-empty size target to enable keeping non-pinned stuff around
     let mut store = BlockStore::memory(
         Config::default()
-            .with_size_targets(SizeTargets::new(10, 10000))
+            .with_size_targets(10, 10000)
             .with_cache_tracker(tracker),
     )?;
 
@@ -372,18 +381,18 @@ fn test_reverse_alias() -> anyhow::Result<()> {
     let block = pinned(0);
     assert_eq!(store.reverse_alias(block.cid())?, None);
     store.put_block(&block, None)?;
-    assert_eq!(store.reverse_alias(block.cid())?, Some(vec![]));
+    assert_eq!(store.reverse_alias(block.cid())?, Some(hashset! {}));
     store.alias(&b"leaf"[..], Some(block.cid()))?;
     assert_eq!(
         store.reverse_alias(block.cid())?,
-        Some(vec![b"leaf".to_vec()])
+        Some(hashset! {b"leaf".to_vec()})
     );
     let block2 = links("1", vec![&block]); // needs link to cid
     store.put_block(&block2, None)?;
     store.alias(&b"root"[..], Some(block2.cid()))?;
     assert_eq!(
         store.reverse_alias(block.cid())?,
-        Some(vec![b"leaf".to_vec(), b"root".to_vec()])
+        Some(hashset! {b"leaf".to_vec(), b"root".to_vec()})
     );
     Ok(())
 }
@@ -422,13 +431,13 @@ fn temp_pin() -> anyhow::Result<()> {
     let mut store = BlockStore::memory(Config::default())?;
     let a = block("a");
     let b = block("b");
-    let alias = store.temp_pin();
+    let mut alias = store.temp_pin();
 
-    store.put_block(&a, Some(&alias))?;
+    store.put_block(&a, Some(&mut alias))?;
     store.gc()?;
     assert!(store.has_block(a.cid())?);
 
-    store.put_block(&b, Some(&alias))?;
+    store.put_block(&b, Some(&mut alias))?;
     store.gc()?;
     assert!(store.has_block(b.cid())?);
 
@@ -445,29 +454,31 @@ fn broken_db() -> anyhow::Result<()> {
     let tmp = TempDir::new("broken_db")?;
     let path = tmp.path().join("mini.sqlite");
     std::fs::copy("test-data/mini.sqlite", &path)?;
-    let store = BlockStore::open_path(DbPath::File(path), Config::default())?;
+    let mut store = BlockStore::open_path(DbPath::File(path), Config::default())?;
     assert!(store.integrity_check().is_ok());
 
     let path = tmp.path().join("broken.sqlite");
     std::fs::copy("test-data/broken.sqlite", &path)?;
-    let store = BlockStore::open_path(DbPath::File(path), Config::default())?;
+    let mut store = BlockStore::open_path(DbPath::File(path), Config::default())?;
     assert!(store.integrity_check().is_err());
     Ok(())
 }
 
 #[test]
-fn shared_file() -> anyhow::Result<()> {
-    let tmp = TempDir::new("shared_file")?;
+fn shared_file() {
+    let tmp = TempDir::new("shared_file").unwrap();
     let path = tmp.path().join("test.sqlite");
-    let mut db1 = BlockStore::open_path(DbPath::File(path.clone()), Config::default())?;
-    let mut db2 = BlockStore::open_path(DbPath::File(path), Config::default())?;
+    let mut db1 = BlockStore::open_path(DbPath::File(path.clone()), Config::default()).unwrap();
+    let mut db2 = BlockStore::open_path(DbPath::File(path), Config::default()).unwrap();
 
     for i in 0..10 {
         let block = block(&format!("block-{}", i));
-        db1.put_block(&block, None)?;
-        assert_eq!(db2.get_block(block.cid())?, Some(block.data().to_vec()));
+        db1.put_block(&block, None).unwrap();
+        assert_eq!(
+            db2.get_block(block.cid()).unwrap(),
+            Some(block.data().to_vec())
+        );
     }
-    Ok(())
 }
 
 #[test]
